#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include <runone.h>
#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Button.H>
#include <FL/Fl_Box.H>
#include <FL/x.H>
#include <FL/filename.H>
#include "LaunchPad.h"
#include <errno.h>
#include <sys/wait.h>

extern void fork_exec(const char* filename, const char* arg1, const char* arg2);
extern void on_screen(const char* s);
extern void show_menu();

#define VR_SOUND_BUTTON 1

void vrsound(int sound)
{
	XKeyboardControl vals;
	vals.bell_pitch = 440;
	vals.bell_duration = 5;
	XChangeKeyboardControl(fl_display, KBBellPitch | KBBellDuration, &vals);
	XBell(fl_display, 100);
}

void LaunchPad::cb(Fl_Widget* w, void* v)
{
	vrsound(VR_SOUND_BUTTON);
	if (v == 0)
		show_menu();
	else {
		LaunchPadItem* lpi = reinterpret_cast<LaunchPadItem*>(v);
		on_screen(lpi->label);
		char command[1024];
		filename_expand(command, lpi->command);
		char* argv[2] = {command, 0};
		runone(argv);
	}
}

void LaunchPad::add_button(int x, char* pm[], LaunchPadItem* lpi,
	int shift, const char* label)
{
	Fl_Button* b = new Fl_Button(x + shift, 1,
		button_width, button_height, label);
	b->callback(cb, (void*)lpi);
}

void no_box(int, int, int, int, Fl_Color)
{
}

void dot_startup()
{
	// Run each line in ~/.wmx/.startup.

	char filename[256];
	filename_expand(filename, STARTUP_PATH);

	FILE* fp = fopen(filename, "r");
	if(fp == NULL)
		return;

	char line[128];
	while(fgets(line, sizeof(line), fp)) {
		char filename[128] = "";
		char arg1[128] = "";
		sscanf(line, "%s %s\n", filename, arg1);
		char* argv[3] = {filename, arg1, 0};
		if(!strlen(arg1))
			argv[1] = 0;
		// Avoids trouble with blank lines
		if (strlen(argv[0])) {
                   fprintf(stderr, "vrsm: runone %s %s\n", argv[0], argv[1]);
		   runone(argv);
		}
	}
	fclose(fp);
}

int orientation = 0, execLaunchpad = 0;
LaunchPad::LaunchPad()
{
	// Check if ~/.wmx/.launchpad exists and has execute permissions
        int launchpadIsexec; 
        {
          struct stat filestat; char launchpadPath[256];
	  filename_expand(launchpadPath, LAUNCHPAD_PATH);
	  if (stat(launchpadPath, &filestat)!=0) perror(LAUNCHPAD_PATH);
          else execLaunchpad = (filestat.st_mode & (S_IXUSR|S_IXGRP|S_IXOTH));
//        fprintf(stderr, "execLaunchpad=%d\n",execLaunchpad);
        }

	// Check orientation
	FILE *fp = fopen(ORIENTATION_PATH, "r");
	if (fp) {
	  fscanf(fp, "%d", &orientation);
	  fclose(fp);
	}

	// launch the window manager
  	fork_exec(VRWM_PATH, "-cursor", "255");

	dot_startup();

	Fl::set_boxtype(FL_FREE_BOXTYPE, no_box, 0, 0, 0, 0);

	if (orientation == 0) {  // Normal orientation
	  // This is tweaked for the VR3[r].
	  int bar_offset = -10;             // VR button is smaller
	  int bar_width = Fl::w() + 8;      // VR button is smaller
	  button_width = bar_width / 7;
	  button_height = 25;

	  _w = new Fl_Window(0, Fl::h()+1, Fl::w(), button_height);

	  _w->clear_border();
	  _w->color(FL_GRAY);

	  // TODO: use pack group
          if (execLaunchpad)
            add_button(bar_offset + (button_width * 0), 0,
              new LaunchPadItem(LAUNCHPAD_LAB,LAUNCHPAD_PATH),0,LAUNCHPAD_SYM);
          else
            add_button(bar_offset + (button_width * 0), 0, 0, 0, LAUNCHPAD_SYM);
	  add_button(bar_offset + (button_width * 1), 0,
            new LaunchPadItem(CONTACTS_LAB, CONTACTS_PATH), 0, CONTACTS_SYM);
	  add_button(bar_offset + (button_width * 2), 0,
            new LaunchPadItem(TODO_LAB, TODO_PATH), 0, TODO_SYM);
	  add_button(bar_offset + (button_width * 3), 0,
	    new LaunchPadItem(SCHEDULE_LAB, SCHEDULE_PATH), 0, SCHEDULE_SYM);
	  add_button(bar_offset + (button_width * 4), 0,
	    new LaunchPadItem(NOTES_LAB, NOTES_PATH), 0, NOTES_SYM);
	  add_button(bar_offset + (button_width * 5), 0,
	    new LaunchPadItem(CALCULATOR_LAB,CALCULATOR_PATH),0,CALCULATOR_SYM);
	  add_button(bar_offset + (button_width * 6), 0,
            new LaunchPadItem(KEYBOARD_LAB, KEYBOARD_PATH), 0, KEYBOARD_SYM);

	  _w->set_non_modal();
	  _w->end();
	}
	else if (orientation == 270) {  // clockwise-> power button down
	  int bar_offset = 0;
	  int bar_height = Fl::h() + 8;    // VR button is smaller
	  button_height = bar_height / 7;
	  button_width = 25;

	  _w = new Fl_Window(Fl::w()+1, 0, button_width, Fl::h());

	  _w->clear_border();
	  _w->color(FL_GRAY);

	  // TODO: use pack group
	  Fl_Button* b;
	  b = new Fl_Button(0, bar_offset + (button_height * 0),
			    button_width, button_height, KEYBOARD_SYM);
	  b->callback(cb, (void*)new LaunchPadItem(KEYBOARD_LAB,KEYBOARD_PATH));
	  b = new Fl_Button(0, bar_offset + (button_height * 1),
			    button_width, button_height, CALCULATOR_SYM);
	  b->callback(cb, 
                     (void*)new LaunchPadItem(CALCULATOR_LAB, CALCULATOR_PATH));
	  b = new Fl_Button(0, bar_offset + (button_height * 2),
			    button_width, button_height, NOTES_SYM);
	  b->callback(cb, (void*)new LaunchPadItem(NOTES_LAB, NOTES_PATH));
	  b = new Fl_Button(0, bar_offset + (button_height * 3),
			    button_width, button_height, SCHEDULE_SYM);
	  b->callback(cb, (void*)new LaunchPadItem(SCHEDULE_LAB,SCHEDULE_PATH));
	  b = new Fl_Button(0, bar_offset + (button_height * 4),
			    button_width, button_height, TODO_SYM);
	  b->callback(cb, (void*)new LaunchPadItem(TODO_LAB, TODO_PATH));
	  b = new Fl_Button(0, bar_offset + (button_height * 5),
			    button_width, button_height, CONTACTS_SYM);
	  b->callback(cb, (void*)new LaunchPadItem(CONTACTS_LAB,CONTACTS_PATH));
	  b = new Fl_Button(0, bar_offset + (button_height * 6),
			       button_width, button_height, LAUNCHPAD_SYM);
          if (execLaunchpad) 
             b->callback(cb, 
                  (void*)new LaunchPadItem(LAUNCHPAD_LAB, LAUNCHPAD_PATH));
	  else b->callback(cb, (void*)0);

	  _w->set_non_modal();
	  _w->end();
	}
	else if (orientation == 90) {  // counter clockwise-> power button up
	  int bar_offset = -10;             // ?
	  int bar_height = Fl::h() + 8;     // ?
	  button_height = bar_height / 7;
	  button_width = 25;

	  _w = new Fl_Window(-1-button_width, 0, button_width, Fl::h());

	  _w->clear_border();
	  _w->color(FL_GRAY);

	  // TODO: use pack group
	  Fl_Button* b = new Fl_Button(0, bar_offset,
			     button_width, button_height, LAUNCHPAD_SYM);
          if (execLaunchpad)
	     b->callback(cb, 
                  (void*)new LaunchPadItem(LAUNCHPAD_LAB, LAUNCHPAD_PATH));
          else b->callback(cb, (void*)0);
	  b = new Fl_Button(0, bar_offset + (button_height * 1),
			    button_width, button_height, CONTACTS_SYM);
	  b->callback(cb, (void*)new LaunchPadItem(CONTACTS_LAB,CONTACTS_PATH));
	  b = new Fl_Button(0, bar_offset + (button_height * 2),
			    button_width, button_height, TODO_SYM);
	  b->callback(cb, (void*)new LaunchPadItem(TODO_LAB, TODO_PATH));
	  b = new Fl_Button(0, bar_offset + (button_height * 3),
			    button_width, button_height, SCHEDULE_SYM);
	  b->callback(cb, (void*)new LaunchPadItem(SCHEDULE_LAB,SCHEDULE_PATH));
	  b = new Fl_Button(0, bar_offset + (button_height * 4),
			    button_width, button_height, NOTES_SYM);
	  b->callback(cb, (void*)new LaunchPadItem(NOTES_LAB, NOTES_PATH));
	  b = new Fl_Button(0, bar_offset + (button_height * 5),
			    button_width, button_height, CALCULATOR_SYM);
	  b->callback(cb, 
                     (void*)new LaunchPadItem(CALCULATOR_LAB, CALCULATOR_PATH));
	  b = new Fl_Button(0, bar_offset + (button_height * 6),
			    button_width, button_height, KEYBOARD_SYM);
	  b->callback(cb, (void*)new LaunchPadItem(KEYBOARD_LAB,KEYBOARD_PATH));

	  _w->set_non_modal();
	  _w->end();
	}
	else /* if (orientation == 180) */ {  // inverted: cable up
	  int bar_offset = 0;
	  int bar_width = Fl::w() + 8;
	  button_width = bar_width / 7;
	  button_height = 25;

	  _w = new Fl_Window(0, -1-button_height, Fl::w(), button_height);

	  _w->clear_border();
	  _w->color(FL_GRAY);

	  // TODO: use pack group
          if (execLaunchpad)
            add_button(bar_offset + (button_width * 6), 0,
             new LaunchPadItem(LAUNCHPAD_LAB, LAUNCHPAD_PATH),0, LAUNCHPAD_SYM);
          else add_button(bar_offset + (button_width * 6),0,0,0, LAUNCHPAD_SYM);
	  add_button(bar_offset + (button_width * 5), 0,
            new LaunchPadItem(CONTACTS_LAB, CONTACTS_PATH), 0, CONTACTS_SYM);
	  add_button(bar_offset + (button_width * 4), 0,
            new LaunchPadItem(TODO_LAB, TODO_PATH), 0, TODO_SYM);
	  add_button(bar_offset + (button_width * 3), 0,
            new LaunchPadItem(SCHEDULE_LAB, SCHEDULE_PATH), 0, SCHEDULE_SYM);
	  add_button(bar_offset + (button_width * 2), 0,
            new LaunchPadItem(NOTES_LAB, NOTES_PATH), 0, NOTES_SYM);
	  add_button(bar_offset + (button_width * 1), 0,
            new LaunchPadItem(CALCULATOR_LAB,CALCULATOR_PATH),0,CALCULATOR_SYM);
	  add_button(bar_offset + (button_width * 0), 0,
            new LaunchPadItem(KEYBOARD_LAB, KEYBOARD_PATH), 0, KEYBOARD_SYM);

	  _w->set_non_modal();
	  _w->end();
	}

	// Cause override-direct to be set.
	// Fl::grab(&dw);
	Fl::grab_ = (Fl_Window*)1;

	_w->show();

	Fl::grab(0);
}
