#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <ctype.h>
#include <sys/types.h>
#include <dirent.h>
#include <sys/stat.h>
#include <unistd.h>
#include <sys/wait.h>
#include <errno.h>

#include "config.h"
#include "wmx.h"

#if WMX_MENU_ITEMS

// Store null-separated list of dirs/files/links
// that make up a menu.
static char* wmxbuffer = NULL;
static int wmxbufsize = 0;
static int num_wmx = 0;

static int scan_wmx_dir(char* path, int bufindex, int nest)
{
	DIR* dir = opendir(path);
	struct stat st;
	int pathlen = strlen (path);
	if(dir) {
		struct dirent* ent;
		while((ent=readdir(dir))) {
			if (ent->d_name[0] == '.')
				continue;
			strcpy(path + pathlen, ent->d_name);
			if (stat(path, &st) < 0)
				continue;
			int len = pathlen + strlen(ent->d_name);
			// worst-case alloc needs
			if (bufindex+len+nest+1 > wmxbufsize)
				wmxbuffer = (char*)realloc(wmxbuffer,
					(wmxbufsize+=1024));

			// Add slashes to represent nesting.
			for (int i = 0; i < nest; i++)
				// extra slash marks menu titles
				wmxbuffer[bufindex++] = '/';

			if (S_ISDIR(st.st_mode)
				&& (st.st_mode & 0555)
				&& nest < MAX_NESTING_DEPTH) {
				// Recurse on the dir.
				strcpy(wmxbuffer + bufindex, path);
				bufindex += len+1;
				strcat(path, "/");
				bufindex = scan_wmx_dir(path, bufindex, nest+1);
				num_wmx++;
				continue;
			}

			if (S_ISREG(st.st_mode) && (st.st_mode & 0111)) {
				// Insert a tilde right before the filename,
				// in order to achieve the sort order
				// of submenus before files.
				char* p = wmxbuffer + bufindex;
				char* slash = strrchr(path, '/');
				strncpy(p, path, slash - path);
				p += (slash - path);
				strcpy(p, "/~");
				strcat(p, slash + 1);
				bufindex += len + 1 + 1;
				num_wmx++;
			}
		}
		closedir(dir);
	}
	return bufindex;
}

// Comparison for qsort
// We keep submenus together by noting that they're proper superstrings.
static int wmxCompare(const void* a, const void* b)
{
	const char* pa = *(const char**)a;
	const char* pb = *(const char**)b;

	// Don't look at leading slashes.
	pa += strspn(pa, "/");
	pb += strspn(pb, "/");

	return strcasecmp(pa, pb);
}

int load_wmx(char*** wmxlist, int wmxlistsize, int* wmx_pathlen, int* pnum_wmx)
{
	static time_t wmx_time = -1;

	// wmxlist is an array of char* pointers
	// (for efficient sorting purposes), 
	// which are stored in mxbuffer (for memory efficiency and to avoid
	// freeing and fragmentation)

	const char* home = getenv("HOME");
	if(!home)
		home = ".";
	char path[1024];
	strcpy(path, home);
	if(path[strlen(path) - 1] != '/')
		strcat(path, "/");
	strcat(path, ".wmx/");
	struct stat st;
	int result = stat(path, &st);
	if (result < 0) {
		fprintf(stderr, "vrsm: load_wmx stat %s: %s\n",
			path, strerror(errno));
		return -1;
	}

#if 0
	// It seems that jffs doesn't update mtime of a directory
	// when the directory's contents change.
	// Disable the optimization for now.

	// Didn't the menu directory change since last time?
	if (st.st_mtime == wmx_time)
		return -2;
#endif

	// Scan the menu directory.
	wmx_time = st.st_mtime;
	num_wmx = 0;
	*wmx_pathlen = strlen(path);
	scan_wmx_dir(path, 0, 0);

	// Not enough room for all the pointers?
	if(num_wmx > wmxlistsize) {
		if (*wmxlist)
			delete[] *wmxlist;
		*wmxlist = new char*[num_wmx];
		wmxlistsize = num_wmx;
	}

	// Make wmxlist.
	char* cmd = wmxbuffer;
	for (int j = 0; j < num_wmx; j++) {
		(*wmxlist)[j] = cmd;
		cmd += strlen(cmd) + 1;
	}
	
	qsort(*wmxlist, num_wmx, sizeof(char*), wmxCompare);

	// Remove the "sort tilde".
	for (int j = 0; j < num_wmx; j++) {
		char* tilde = strrchr((*wmxlist)[j], '~');
		if(tilde)
			strcpy(tilde, tilde + 1);
	}

	*pnum_wmx = num_wmx;

	return 0;
}

#endif
